
#include "keys.h"

//  
#define KEY_DDR               DDRD
#define KEY_PORT              PORTD
#define KEY_PIN               PIND
#define KEY_PIN_SHOT          PD3
#define KEY_PIN_SHOT_DELAYED  PD2

// 
#define PRESSED_KEY_SHOT         (1<<KEY_PIN_SHOT)
#define PRESSED_KEY_SHOT_DELAYED (1<<KEY_PIN_SHOT_DELAYED)
#define PRESSED_KEYS_FROM_PORT   ((~KEY_PIN) & (PRESSED_KEY_SHOT | PRESSED_KEY_SHOT_DELAYED))

//      
typedef enum 
{
Wait_Press   = 0,
Pressed      = 1,
Run_Command  = 2,
Wait_Release = 3
} State_Enum;

//   
static State_Enum State = Wait_Press;
//   / 
static u16 Timer=0;
//   
//     / , ..   
static u8 Pressed_Keys=0;


/****************************/
/*       */
/****************************/
void Key_Init (void)
{
	KEY_DDR &= ~((1<<KEY_PIN_SHOT)|(1<<KEY_PIN_SHOT_DELAYED));	//  = 
	KEY_PORT |= ((1<<KEY_PIN_SHOT)|(1<<KEY_PIN_SHOT_DELAYED));	//   
}


/******************************************/
/*          */
/******************************************/
void Key_Process (void)
{
	//     ,     .
	//    ,     ,   .
	//          (,   ).
	//      ,    (State).
	
	//      
	//    -  
	if (Pressed_Keys != PRESSED_KEYS_FROM_PORT) State=Wait_Press;
	//  
	switch (State)
	{
		case Wait_Press:
		{
			Timer=0;
			if (Pressed_Keys) State=Pressed;
		}; break;
		case Pressed:
		{
			Timer++;
			if (Pressed_Keys==0) State=Wait_Release;
			if (Timer>(CALC_TICKS(KEY_PRESS_INTERVAL))) State=Run_Command;
		}; break;
		case Run_Command:
		{
			if (Pressed_Keys == PRESSED_KEY_SHOT) IR_Shot();
			if (Pressed_Keys == PRESSED_KEY_SHOT_DELAYED) IR_Shot_Delayed();
			State=Wait_Release;
		}; break;
		case Wait_Release:
		{
			if (Pressed_Keys==0) State=Wait_Press;
		}; break;
	}
	//    
	Pressed_Keys = PRESSED_KEYS_FROM_PORT;
	//    ,   
	if (Pressed_Keys) Sleep_Reset();
}


/*****************************************************************************/
/*      INT0                                     */
/*     ,          */
/*****************************************************************************/
ISR (INT0_vect)
{
	asm volatile ("nop");
}


/*****************************************************************************/
/*      INT1                                     */
/*     ,          */
/*****************************************************************************/
ISR (INT1_vect)
{
	asm volatile ("nop");
}

